@@ -1,5 +1,5 @@
 /* test-lookup.c --- Self tests for IDNA processing
-   Copyright (C) 2011-2016 Simon Josefsson
+   Copyright (C) 2011-2017 Simon Josefsson
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -783,30 +783,40 @@ static const struct idna idna[] = {
   },
   {"\xe2\x84\xab", "", IDN2_NOT_NFC},
   {"\xe2\x84\xa6", "", IDN2_NOT_NFC},
+  {
+    /* blåbærgrød.no composed */
+    "\x62\x6c\xc3\xa5\x62\xc3\xa6\x72\x67\x72\xc3\xb8\x64\x2e\x6e\x6f", "xn--blbrgrd-fxak7p.no", IDN2_OK
+  },
+  {
+    /* blåbærgrød.no partially decomposed */
+    "\x62\x6c\x61\xcc\x8a\x62\xc3\xa6\x72\x67\x72\xc3\xb8\x64\x2e\x6e\x6f", "", IDN2_NOT_NFC
+  },
+  {
+    /* blåbærgrød.no partially decomposed */
+    "\x62\x6c\x61\xcc\x8a\x62\xc3\xa6\x72\x67\x72\xc3\xb8\x64\x2e\x6e\x6f", "xn--blbrgrd-fxak7p.no", IDN2_OK, IDN2_NFC_INPUT
+  },
+  {
+    /* blåbærgrød.no partially decomposed */
+    "\x62\x6c\x61\xcc\x8a\x62\xc3\xa6\x72\x67\x72\xc3\xb8\x64\x2e\x6e\x6f", "xn--blbrgrd-fxak7p.no", IDN2_OK, IDN2_TRANSITIONAL
+  },
+  {
+    /* blåbærgrød.no partially decomposed */
+    "\x62\x6c\x61\xcc\x8a\x62\xc3\xa6\x72\x67\x72\xc3\xb8\x64\x2e\x6e\x6f", "xn--blbrgrd-fxak7p.no", IDN2_OK, IDN2_NONTRANSITIONAL
+  },
+  {
+    /* 0x00FFFFFF, character with 5 bytes UTF-8 representation */
+    "\xf8\xbf\xbf\xbf\xbf", "", IDN2_ENCODING_ERROR, IDN2_NONTRANSITIONAL
+  },
+  {
+    /* bad utf-8 encoding */
+    "\x7e\x64\x61\x72\x10\x2f\x2f\xf9\x2b\x71\x60\x79\x7b\x2e\x63\x75\x2b\x61\x65\x72\x75\x65\x56\x66\x7f\x62\xc5\x76\xe5\x00",
+    "", IDN2_ENCODING_ERROR, IDN2_NONTRANSITIONAL,
+  },
 };
 
-static int debug = 1;
 static int ok = 0, failed = 0;
 static int break_on_error = 0;
 
-static void
-hexprint (const char *str, size_t len)
-{
-  size_t i;
-
-  printf ("\t;; ");
-  if (str && len)
-    for (i = 0; i < len; i++)
-      {
-	printf ("%02x ", (str[i] & 0xFF));
-	if ((i + 1) % 8 == 0)
-	  printf (" ");
-	if ((i + 1) % 16 == 0 && i + 1 < len)
-	  printf ("\n\t;; ");
-      }
-  printf ("\n");
-}
-
 static char *_nextField(char **line)
 {
   char *s = *line, *e;
@@ -865,14 +875,16 @@ static int _scan_file(const char *fname, int(*scan)(char *))
 static void
 test_homebrewed(void)
 {
+  uint32_t dummy_u32[4] = { 'a', 'b', 'c', 0 };
   uint8_t *out;
+  char outbuf[4];
   size_t i;
   int rc;
 
   for (i = 0; i < sizeof (idna) / sizeof (idna[0]); i++)
     {
-      rc = idn2_lookup_u8 (idna[i].in, &out, idna[i].flags);
-      printf ("%3d  %-25s %-40s %s\n", i, idn2_strerror_name (rc),
+      rc = idn2_lookup_u8 ((uint8_t *) idna[i].in, &out, idna[i].flags);
+      printf ("%3d  %-25s %-40s %s\n", (int) i, idn2_strerror_name (rc),
 	      rc == IDN2_OK ? idna[i].out : "", idna[i].in);
 
       if (rc != idna[i].rc && rc == IDN2_ENCODING_ERROR) {
@@ -880,7 +892,7 @@ test_homebrewed(void)
       } else if (rc != idna[i].rc && idna[i].rc != -1) {
 	failed++;
 	printf("expected rc %d got rc %d\n", idna[i].rc, rc);
-      } else if (rc == IDN2_OK && strcmp (out, idna[i].out) != 0) {
+      } else if (rc == IDN2_OK && strcmp ((char *) out, idna[i].out) != 0) {
 	failed++;
 	printf("expected: %s\ngot: %s\n", idna[i].out, out);
       } else
@@ -922,9 +934,47 @@ test_homebrewed(void)
     printf("special #4 failed with %d\n", rc);
   } else if (out) {
     failed++;
-    printf("special #2 failed with out!=NULL\n");
+    printf("special #4 failed with out!=NULL\n");
   } else
     ok++;
+
+  if ((rc = idn2_lookup_ul ("abc", NULL, 0)) != IDN2_OK) {
+    failed++;
+    printf("special #5 failed with %d\n", rc);
+  } else
+    ok++;
+
+  /* test libidn compatibility functions */
+  if ((rc = idna_to_ascii_lz ("abc", (char **) &out, 0)) != IDN2_OK) {
+    failed++;
+    printf("special #6 failed with %d\n", rc);
+  } else {
+    idna_free (out);
+    ok++;
+  }
+
+  if ((rc = idna_to_ascii_8z ("abc", (char **) &out, 0)) != IDN2_OK) {
+    failed++;
+    printf("special #7 failed with %d\n", rc);
+  } else {
+    idna_free (out);
+    ok++;
+  }
+
+  if ((rc = idna_to_ascii_4z (dummy_u32, (char **) &out, 0)) != IDN2_OK) {
+    failed++;
+    printf("special #8 failed with %d\n", rc);
+  } else {
+    idna_free (out);
+    ok++;
+  }
+
+  if ((rc = idna_to_ascii_4i (dummy_u32, 4, outbuf, 0)) != IDN2_OK) {
+    failed++;
+    printf("special #9 failed with %d\n", rc);
+  } else {
+    ok++;
+  }
 }
 
 // decode embedded UTF-16/32 sequences
@@ -982,7 +1032,7 @@ _decodeIdnaTest(uint8_t *src_u8)
     return NULL;
   }
 
-  return src_u8;
+  return (char *) src_u8;
 }
 
 static void
@@ -991,7 +1041,7 @@ _check_toASCII(char *source, char *expected, int transitional, int expected_toAS
   int rc;
   char *ace = NULL;
 
-  rc = idn2_lookup_u8 (source, (uint8_t **) &ace, transitional ? IDN2_TRANSITIONAL : IDN2_NONTRANSITIONAL);
+  rc = idn2_lookup_u8 ((uint8_t *) source, (uint8_t **) &ace, transitional ? IDN2_TRANSITIONAL : IDN2_NONTRANSITIONAL);
 
   // printf("n=%d expected=%s t=%d got=%s, expected_failure=%d\n", n, expected, transitional, ace ? ace : "", expected_toASCII_failure);
   if (rc && expected_toASCII_failure) {
@@ -1029,7 +1079,7 @@ test_IdnaTest(char *linep)
 
   // sigh, these Unicode people really mix UTF-8 and UCS-2/4
   // quick and dirty translation of '\uXXXX' found in IdnaTest.txt including surrogate handling
-  source = _decodeIdnaTest(source);
+  source = _decodeIdnaTest((uint8_t *) source);
   if (!source)
     return 0; // some Unicode sequences can't be encoded into UTF-8, skip them
 
@@ -1079,16 +1129,52 @@ separator(void)
 	"-------------------------------------");
 }
 
+static void
+test_unicode_range (void)
+{
+  uint32_t i, ucs4[2];
+  uint8_t *utf8, *out;
+  size_t len;
+  int rc;
+
+  /* Unicode range is 0-0x10FFFF, go a bit further */
+  for (i = 0; i < 0x11FFFF; i++)
+  {
+    ucs4[0] = i;
+    ucs4[1] = 0;
+
+    utf8 = u32_to_u8 (ucs4, 2, NULL, &len);
+
+    rc = idn2_lookup_u8 (utf8, &out, 0);
+    if (rc == IDN2_OK)
+      idn2_free (out);
+
+    rc = idn2_lookup_u8 (utf8, &out, IDN2_NFC_INPUT);
+    if (rc == IDN2_OK)
+      idn2_free (out);
+
+    rc = idn2_lookup_u8 (utf8, &out, IDN2_TRANSITIONAL);
+    if (rc == IDN2_OK)
+      idn2_free (out);
+
+    rc = idn2_lookup_u8 (utf8, &out, IDN2_NONTRANSITIONAL);
+    if (rc == IDN2_OK)
+      idn2_free (out);
+
+    free (utf8);
+  }
+}
+
 int
 main (int argc, const char *argv[])
 {
-  separator();
+  separator ();
   puts ("                                          IDNA2008 Lookup\n");
   puts ("  #  Result                    ACE output                  "
 	"             Unicode input");
   separator();
 
-  test_homebrewed();
+  test_homebrewed ();
 
   separator();
 
@@ -1098,6 +1184,8 @@ main (int argc, const char *argv[])
 
   separator();
 
+  test_unicode_range ();
+
   if (failed) {
     printf("Summary: %d out of %d tests failed\n", failed, ok + failed);
     return EXIT_FAILURE;
