@@ -1,5 +1,5 @@
 /* test-punycode.c --- Self tests for Libidn2 punycode.
-   Copyright (C) 2002-2016 Simon Josefsson
+   Copyright (C) 2002-2017 Simon Josefsson
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@ struct punycode
   int rc;
 };
 
-const struct punycode punycode[] = {
+static const struct punycode punycode[] = {
   {
    "(A) Arabic (Egyptian)", 17,
    {
@@ -162,11 +162,11 @@ const struct punycode punycode[] = {
     0x0020, 0x003C, 0x002D}, "-> $1.00 <--", IDN2_OK}
 };
 
-int debug = 0;
-int error_count = 0;
-int break_on_error = 0;
+static int debug = 0;
+static int error_count = 0;
+static int break_on_error = 0;
 
-void
+_GL_ATTRIBUTE_FORMAT_PRINTF (1, 2) static void
 fail (const char *format, ...)
 {
   va_list arg_ptr;
@@ -179,7 +179,7 @@ fail (const char *format, ...)
     exit (EXIT_FAILURE);
 }
 
-void
+static void
 ucs4print (const uint32_t * str, size_t len)
 {
   size_t i;
@@ -226,7 +226,7 @@ main (void)
   for (i = 0; i < sizeof (punycode) / sizeof (punycode[0]); i++)
     {
       if (debug)
-	printf ("PUNYCODE entry %d: %s\n", i, punycode[i].name);
+	printf ("PUNYCODE entry %d: %s\n", (int) i, punycode[i].name);
 
       if (debug)
 	{
@@ -235,11 +235,11 @@ main (void)
 	}
 
       outlen = BUFSIZ;
-      rc = _idn2_punycode_encode (punycode[i].inlen, punycode[i].in,
-				    NULL, &outlen, p);
+      rc = _idn2_punycode_encode (
+	punycode[i].inlen, punycode[i].in, &outlen, p);
       if (rc != punycode[i].rc)
 	{
-	  fail ("punycode_encode() entry %d failed: %d\n", i, rc);
+	  fail ("punycode_encode() entry %d failed: %d\n", (int) i, rc);
 	  if (debug)
 	    printf ("FATAL\n");
 	  continue;
@@ -261,7 +261,7 @@ main (void)
 	  if (strlen (punycode[i].out) != strlen (p) ||
 	      memcmp (punycode[i].out, p, strlen (p)) != 0)
 	    {
-	      fail ("punycode() entry %d failed\n", i);
+	      fail ("punycode() entry %d failed\n", (int) i);
 	      if (debug)
 		printf ("ERROR\n");
 	    }
@@ -278,10 +278,10 @@ main (void)
 
       outlen = BUFSIZ;
       rc = _idn2_punycode_decode (strlen (punycode[i].out), punycode[i].out,
-				    &outlen, q, NULL);
+				    &outlen, q);
       if (rc != punycode[i].rc)
 	{
-	  fail ("punycode() entry %d failed: %d\n", i, rc);
+	  fail ("punycode() entry %d failed: %d\n", (int) i, rc);
 	  if (debug)
 	    printf ("FATAL\n");
 	  continue;
@@ -302,7 +302,7 @@ main (void)
 	  if (punycode[i].inlen != outlen ||
 	      memcmp (punycode[i].in, q, outlen) != 0)
 	    {
-	      fail ("punycode_decode() entry %d failed\n", i);
+	      fail ("punycode_decode() entry %d failed\n", (int) i);
 	      if (debug)
 		printf ("ERROR\n");
 	    }
